@@ -13,7 +13,6 @@
 
 (define-module (ice-9 format)
   :use-module (ice-9 and-let-star)
-  :use-module (ice-9 threads)
   :autoload (ice-9 pretty-print) (pretty-print)
   :replace (format)
   :export (format:symbol-case-conv
@@ -194,10 +193,6 @@
 	  (let ((arg-pos (format:format-work fmt args))
 		(arg-len (length args)))
 	    (cond
-	     ((< arg-pos arg-len)
-	      (set! format:arg-pos (+ arg-pos 1))
-	      (set! format:pos (string-length fmt))
-	      (format:error "~a superfluous argument~:p" (- arg-len arg-pos)))
 	     ((> arg-pos arg-len)
 	      (set! format:arg-pos (+ arg-len 1))
 	      (display format:arg-pos)
@@ -336,8 +331,8 @@
 					      (next-arg) #f params)
 		       (anychar-dispatch))
 		      ((#\S)			; Slashified -- for parsers
-		       (set! format:read-proof (memq modifier
-						     '(colon colon-at)))
+		       (set! format:read-proof
+			     (memq modifier '(colon colon-at)))
 		       (format:out-obj-padded (memq modifier '(at colon-at))
 					      (next-arg) #t params)
 		       (anychar-dispatch))
@@ -811,132 +806,23 @@
 	    (set! format:pos recursive-pos-save)
 	    arg-pos)))				; return the position in the arg. list
 
-       ;; format:obj->str returns a R4RS representation as a string of an arbitrary
-       ;; scheme object.
-       ;; First parameter is the object, second parameter is a boolean if the
-       ;; representation should be slashified as `write' does.
-       ;; It uses format:char->str which converts a character into
-       ;; a slashified string as `write' does and which is implementation dependent.
-       ;; It uses format:iobj->str to print out internal objects as
-       ;; quoted strings so that the output can always be processed by (read)
+       ;; when format:read-proof is true, format:obj->str will wrap
+       ;; result strings starting with "#<" in an extra pair of double
+       ;; quotes.
+       
+       (format:read-proof #f)
+
+       ;; format:obj->str returns a R4RS representation as a string of
+       ;; an arbitrary scheme object.
 
        (format:obj->str
 	(lambda (obj slashify)
-	  (define (obj->str obj slashify visited)
-	    (if (memq obj (cdr visited))
-		(let ((n (- (list-index (cdr visited) (cdr obj)))))
-		  (string-append "#" (number->string n) "#"))
-		(cond
-		 ((string? obj)
-		  (if slashify
-		      (let ((obj-len (string-length obj)))
-			(string-append
-			 "\""
-			 (let loop ((i 0) (j 0)) ; taken from Marc Feeley's pp.scm
-			   (if (= j obj-len)
-			       (string-append (substring obj i j) "\"")
-			       (let ((c (string-ref obj j)))
-				 (if (or (char=? c #\\)
-					 (char=? c #\"))
-				     (string-append (substring obj i j) "\\"
-						    (loop j (+ j 1)))
-				     (loop i (+ j 1))))))))
-		      obj))
-		 
-		 ((boolean? obj) (if obj "#t" "#f"))
-		 
-		 ((number? obj) (number->string obj))
-
-		 ((symbol? obj) 
-		  (if format:symbol-case-conv
-		      (format:symbol-case-conv (symbol->string obj))
-		      (symbol->string obj)))
-		 
-		 ((char? obj)
-		  (if slashify
-		      (format:char->str obj)
-		      (string obj)))
-		 
-		 ((null? obj) "()")
-
-		 ((input-port? obj)
-		  (format:iobj->str obj))
-		 
-		 ((output-port? obj)
-		  (format:iobj->str obj))
-		 
-		 ((pair? obj)
-		  (string-append "("
-				 (let loop ((obj-list obj)
-					    (visited visited)
-					    (offset 0)
-					    (prefix ""))
-				   (cond ((null? (cdr obj-list))
-					  (string-append
-					   prefix
-					   (obj->str (car obj-list)
-						     #t
-						     (cons (car obj-list) visited))))
-					 ((memq (cdr obj-list) visited)
-					  (string-append
-					   prefix
-					   (obj->str (car obj-list)
-						     #t
-						     (cons (car obj-list) visited))
-					   " . #"
-					   (number->string
-					    (- offset
-					       (list-index visited (cdr obj-list))))
-					   "#"))
-					 ((pair? (cdr obj-list))
-					  (loop (cdr obj-list)
-						(cons (cdr obj-list) visited)
-						(+ 1 offset)
-						(string-append
-						 prefix
-						 (obj->str (car obj-list)
-							   #t
-							   (cons (car obj-list) visited))
-						 " ")))
-					 (else
-					  (string-append
-					   prefix
-					   (obj->str (car obj-list)
-						     #t
-						     (cons (car obj-list) visited))
-					   " . "
-					   (obj->str (cdr obj-list)
-						     #t
-						     (cons (cdr obj-list) visited))))))
-				 ")"))
-
-		 ((vector? obj)
-		  (string-append "#" (obj->str (vector->list obj) #t visited)))
-
-		 (else				; only objects with an #<...> 
-		  (format:iobj->str obj)))))	; representation should fall in here
-	  (obj->str obj slashify (list obj))))
-
-       ;; format:iobj->str reveals the implementation dependent representation of 
-       ;; #<...> objects with the use of display and call-with-output-string.
-       ;; If format:read-proof is set to #t the resulting string is additionally 
-       ;; set into string quotes.
-
-       (format:read-proof #f)
-
-       (format:iobj->str
-	(lambda (iobj)
-	  (if (or format:read-proof
-		  format:iobj-case-conv)
-	      (string-append 
-	       (if format:read-proof "\"" "")
-	       (if format:iobj-case-conv
-		   (format:iobj-case-conv
-		    (call-with-output-string (lambda (p) (display iobj p))))
-		   (call-with-output-string (lambda (p) (display iobj p))))
-	       (if format:read-proof "\"" ""))
-	      (call-with-output-string (lambda (p) (display iobj p))))))
-
+	  (let ((res (if slashify
+			 (object->string obj)
+			 (with-output-to-string (lambda () (display obj))))))
+	    (if (and format:read-proof (string-prefix? "#<" res))
+		(object->string res)
+		res))))
 
        ;; format:char->str converts a character into a slashified string as
        ;; done by `write'. The procedure is dependent on the integer
@@ -1017,7 +903,7 @@
 			(commawidth (format:par pars l 3 3 "commawidth")))
 		    (if mincol
 			(let ((numlen numstr-len)) ; calc. the output len of number
-			  (if (and (memq modifier '(at colon-at)) (> number 0))
+			  (if (and (memq modifier '(at colon-at)) (>= number 0))
 			      (set! numlen (+ numlen 1)))
 			  (if (memq modifier '(colon colon-at))
 			      (set! numlen (+ (quotient (- numstr-len 
@@ -1027,7 +913,7 @@
 			  (if (> mincol numlen)
 			      (format:out-fill (- mincol numlen) padchar))))
 		    (if (and (memq modifier '(at colon-at))
-			     (> number 0))
+			     (>= number 0))
 			(format:out-char #\+))
 		    (if (memq modifier '(colon colon-at)) ; insert comma character
 			(let ((start (remainder numstr-len commawidth))
@@ -1486,44 +1372,49 @@
 		  (width (format:par pars l 2 0 "width"))
 		  (padch (format:par pars l 3 format:space-ch #f)))
 
-	      (format:parse-float
-	       (if (string? number) number (number->string number)) #t 0)
-	      (if (<= (- format:fn-len format:fn-dot) digits)
-		  (format:fn-zfill #f (- digits (- format:fn-len format:fn-dot)))
-		  (format:fn-round digits))
-	      (let ((numlen (+ format:fn-len 1)))
-		(if (or (not format:fn-pos?) (memq modifier '(at colon-at)))
-		    (set! numlen (+ numlen 1)))
+	      (cond
+	       ((or (inf? number) (nan? number))
+		(format:out-inf-nan number width digits #f #f padch))
+
+	       (else
+		(format:parse-float
+		 (if (string? number) number (number->string number)) #t 0)
+		(if (<= (- format:fn-len format:fn-dot) digits)
+		    (format:fn-zfill #f (- digits (- format:fn-len format:fn-dot)))
+		    (format:fn-round digits))
+		(let ((numlen (+ format:fn-len 1)))
+		  (if (or (not format:fn-pos?) (memq modifier '(at colon-at)))
+		      (set! numlen (+ numlen 1)))
+		  (if (and mindig (> mindig format:fn-dot))
+		      (set! numlen (+ numlen (- mindig format:fn-dot))))
+		  (if (and (= format:fn-dot 0) (not mindig))
+		      (set! numlen (+ numlen 1)))
+		  (if (< numlen width)
+		      (case modifier
+			((colon)
+			 (if (not format:fn-pos?)
+			     (format:out-char #\-))
+			 (format:out-fill (- width numlen) (integer->char padch)))
+			((at)
+			 (format:out-fill (- width numlen) (integer->char padch))
+			 (format:out-char (if format:fn-pos? #\+ #\-)))
+			((colon-at)
+			 (format:out-char (if format:fn-pos? #\+ #\-))
+			 (format:out-fill (- width numlen) (integer->char padch)))
+			(else
+			 (format:out-fill (- width numlen) (integer->char padch))
+			 (if (not format:fn-pos?)
+			     (format:out-char #\-))))
+		      (if format:fn-pos?
+			  (if (memq modifier '(at colon-at)) (format:out-char #\+))
+			  (format:out-char #\-))))
 		(if (and mindig (> mindig format:fn-dot))
-		    (set! numlen (+ numlen (- mindig format:fn-dot))))
+		    (format:out-fill (- mindig format:fn-dot) #\0))
 		(if (and (= format:fn-dot 0) (not mindig))
-		    (set! numlen (+ numlen 1)))
-		(if (< numlen width)
-		    (case modifier
-		      ((colon)
-		       (if (not format:fn-pos?)
-			   (format:out-char #\-))
-		       (format:out-fill (- width numlen) (integer->char padch)))
-		      ((at)
-		       (format:out-fill (- width numlen) (integer->char padch))
-		       (format:out-char (if format:fn-pos? #\+ #\-)))
-		      ((colon-at)
-		       (format:out-char (if format:fn-pos? #\+ #\-))
-		       (format:out-fill (- width numlen) (integer->char padch)))
-		      (else
-		       (format:out-fill (- width numlen) (integer->char padch))
-		       (if (not format:fn-pos?)
-			   (format:out-char #\-))))
-		    (if format:fn-pos?
-			(if (memq modifier '(at colon-at)) (format:out-char #\+))
-			(format:out-char #\-))))
-	      (if (and mindig (> mindig format:fn-dot))
-		  (format:out-fill (- mindig format:fn-dot) #\0))
-	      (if (and (= format:fn-dot 0) (not mindig))
-		  (format:out-char #\0))
-	      (format:out-substr format:fn-str 0 format:fn-dot)
-	      (format:out-char #\.)
-	      (format:out-substr format:fn-str format:fn-dot format:fn-len)))))
+		    (format:out-char #\0))
+		(format:out-substr format:fn-str 0 format:fn-dot)
+		(format:out-char #\.)
+		(format:out-substr format:fn-str format:fn-dot format:fn-len)))))))
 
 					; the flonum buffers
 
@@ -1569,8 +1460,8 @@
 				(if (> format:fn-dot left-zeros) 
 				    (begin		; norm 0{0}nn.mm to nn.mm
 				      (format:fn-shiftleft left-zeros)
-				      (set! left-zeros 0)
-				      (set! format:fn-dot (- format:fn-dot left-zeros)))
+				      (set! format:fn-dot (- format:fn-dot left-zeros))
+				      (set! left-zeros 0))
 				    (begin		; normalize 0{0}.nnn to .nnn
 				      (format:fn-shiftleft format:fn-dot)
 				      (set! left-zeros (- left-zeros format:fn-dot))
